//--------------------------------------------------------------//
// Header
//
// Lightworks effects have to have a _LwksEffectInfo block
// which defines basic information about the effect (ie. name
// and category). EffectGroup must be "GenericPixelShader".
//--------------------------------------------------------------//
int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Lift, Gamma, Gain";       // The title
   string Category    = "Colour";                  // Governs the category that the effect appears in in Lightworks
> = 0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//

// For each 'texture' declared here, Lightworks adds a matching
// input to your effect (so for a four input effect, you'd need
// to delcare four textures and samplers)

texture Input;

sampler FgSampler = sampler_state
{
   Texture = <Input>;
};

//--------------------------------------------------------------//
// Define parameters here.
//
// The Lightworks application will automatically generate
// sliders/controls for all parameters which do not start
// with a a leading '_' character
//--------------------------------------------------------------//
float MasterLift
<
   string Description = "Lift";
   string Group       = "Master"; // Causes this parameter to be displayed in a group called 'Master'
   float MinVal       = -1.00;
   float MaxVal       = 1.00;
> = 0.0; // Default value

float MasterGamma
<
   string Description = "Gamma";
   string Group       = "Master";
   float MinVal       = 0.1;
   float MaxVal       = 4.00;
> = 1.0; // Default value

float MasterGain
<
   string Description = "Gain";
   string Group       = "Master";
   float MinVal       = -1.00;
   float MaxVal       = 1.00;
> = 0.0; // Default value

float RedLift
<
   string Description = "Lift";
   string Group       = "Red";
   float MinVal       = -1.00;
   float MaxVal       = 1.00;
> = 0.0; // Default value

float RedGamma
<
   string Description = "Gamma";
   string Group       = "Red";
   float MinVal       = 0.1;
   float MaxVal       = 4.00;
> = 1.0; // Default value

float RedGain
<
   string Description = "Gain";
   string Group       = "Red";
   float MinVal       = -1.00;
   float MaxVal       = 1.00;
> = 0.0; // Default value

float GreenLift
<
   string Description = "Lift";
   string Group       = "Green";
   float MinVal       = -1.00;
   float MaxVal       = 1.00;
> = 0.0; // Default value

float GreenGamma
<
   string Description = "Gamma";
   string Group       = "Green";
   float MinVal       = 0.1;
   float MaxVal       = 4.00;
> = 1.0; // Default value

float GreenGain
<
   string Description = "Gain";
   string Group       = "Green";
   float MinVal       = -1.00;
   float MaxVal       = 1.00;
> = 0.0; // Default value

float BlueLift
<
   string Description = "Lift";
   string Group       = "Blue";
   float MinVal       = -1.00;
   float MaxVal       = 1.00;
> = 0.0; // Default value

float BlueGamma
<
   string Description = "Gamma";
   string Group       = "Blue";
   float MinVal       = 0.1;
   float MaxVal       = 4.00;
> = 1.0; // Default value

float BlueGain
<
   string Description = "Gain";
   string Group       = "Blue";
   float MinVal       = -1.00;
   float MaxVal       = 2.00;
> = 0.0; // Default value

#pragma warning ( disable : 3571 )

//--------------------------------------------------------------
// Pixel Shader
//
// This section defines the code which the GPU will
// execute for every pixel in an output image.
//
// Note that pixels are processed out of order, in parallel.
// Using shader model 2.0, so there's a 64 instruction limit -
// use multple passes if you need more.
//--------------------------------------------------------------
float4 LiftGammaGain( float2 xy : TEXCOORD1 ) : COLOR
{
   // Read a pixel from the source image at position 'xy'
   // and place it in the variable 'source'
   float4 source = tex2D( FgSampler, xy );

   // Set up compound RGBA variables from the user-specified channel parameters
   float4 gamma = { RedGamma, GreenGamma, BlueGamma , 1.0 };
   float4 lift  = { RedLift,  GreenLift,  BlueLift,   1.0 };
   float4 gain  = { RedGain,  GreenGain,  BlueGain,   1.0 };

   gamma.rgb += MasterGamma - 1;
   lift.rgb  += MasterLift;
   gain.rgb  += MasterGain + 1;

   // Clamp the values.  Would be nice to use clamp()/saturate() here, but the
   // run-time FX compiler in VS2005 will fail if you do.  Nice.
   if ( gamma.r < 0.1 ) gamma.r = 0.1;
   if ( gamma.g < 0.1 ) gamma.g = 0.1;
   if ( gamma.b < 0.1 ) gamma.b = 0.1;

   // Perform the correction
   float4 corrected = pow( source, 1 / gamma ) * gain;
   corrected += ( 1 - corrected ) * lift;

   // Honour source alpha
   corrected = lerp( source, corrected, source.a );
   corrected.a = source.a;

   return corrected;
}

//--------------------------------------------------------------
// Technique
//
// Specifies the order of passes (we only have a single pass, so
// there's not much to do)
//--------------------------------------------------------------
technique SampleFxTechnique
{
   pass SinglePass
   {
      PixelShader = compile PROFILE LiftGammaGain();
   }
}

