//-------------------------------------------------------------------------------------------------
// ColourConvert_Subs.fx
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//-------------------------------------------------------------------------------------------------
/*
   Colour Conversion Formulas
   ==========================

   Variables
   ---------
   R, G, B: Normalised values per channel, 0.0 to 1.0
   Y, Cr, Cb: Normalised values per channel, 0.0 to 1.0

   Constants
   ---------
         BT-601(SD)  BT-709(HD)
         -------     -------
   Nr    0.299       0.2126
   Ng    0.587       0.7152
   Nb    0.114       0.0722
   Kr    0.713       0.635
   Kb    0.564       0.539

   To convert from R G B to Y U V:

   Y = ( Nr * R ) + ( Ng * G) + ( Nb * B )

   Cr = Kr * ( R - Y )
      = ( (Kr - ( Kr * Nr ) ) * R ) - ( Kr * Ng * G ) - ( Kr * Nb * B )

   Cb = Kb * ( B - Y )
      = - ( Kb * Nr * R ) - ( Kb * Ng * G ) + ( ( Kb - Kb * Nb ) * B )

   Substituting constants for BT-601:

   Y =  ( 0.299 * R )    + ( 0.587 * G )   + ( 0.114 * B )
   Cr = ( 0.5 * R )      - ( 0.41869 * G ) - ( 0.08131 * B )
   Cb = ( -0.16874 * R ) - ( 0.33126 * G ) + ( 0.5 * B )

   Substituting constants for BT-709:

   Y =  ( 0.2126 * R )    + ( 0.7152 * G )    + ( 0.0722 * B )
   Cr = ( 0.5 * R )       - ( 0.45415 * G )   - ( 0.045847 * B )
   Cb = ( -0.114591 * R ) - ( 0.3854928 * G ) + ( 0.5 * B )

   Note: If R,G and B are in the range 0..1, these YCrCb will be in the range 0..1.
   This is NOT correctly scaled and offset for BT.601 or BT.709.

   To scale in the float domain:

   Y' = ( Y * ( 876.0 / 1023.0 ) ) + ( 64.0 / 256.0 )
      = ( Y * 0.875 ) + 0.0625

   Cr' = ( Cr * ( 899.0 / 1023.0 ) ) + ( 128.0 / 256.0 )
       = ( Cr * 0.875 ) + 0.5

   Cb' = ( Cb * ( 899.0 / 1023.0 ) ) + ( 128.0 / 256.0 )
       = ( Cb * 0.875 ) + 0.5

   To convert from Y U V to R G B:

   R = ( Cr + ( Kr * Y ) ) / Kr

     = Y + ( Cr * ( 1 / Kr ) )

   G = ( Y - ( Nr * R ) - ( Nb * B ) ) / Ng

     = Y * ( 1 - Nr - Nb ) / Ng - Cr * Nr/Kr/Ng - Cb * Nb/Kb/Ng

     NB: ( 1 - Nr - Nb ) / Ng == 1

   B = ( Cb + ( Kb * Y ) ) / Kb

     = Y + ( Cb * ( 1 / Kb ) )

   Substituting constants for BT-601:

   R = Y + 1.402     * Cr
   G = Y - 0.7141363 * Cr - 0.3441363 * Cb
   B = Y                  + 1.772     * Cb

   Substituting constants for BT-709:

   R = Y + 1.5748031 * Cr
   G = Y - 0.4681252 * Cr - 0.1872927 * Cb
   B = Y                  + 1.8552876 * Cb

   The above assume that the input ranges are Y 0..1, Cr and Cb -0.5..+0.5. This does not correspond
   to the values normally used which will, in 10-bit land, be 64-940 for Y, 64-963 for Cr and Cb.

*/

//-------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------- Normal Definitions --
//-------------------------------------------------------------------------------------------------
// "Normal" include stuff which is used in here, and must be included only once.
#ifndef ColourConvert_Subs_fx_INCLUDED
#define ColourConvert_Subs_fx_INCLUDED

   // Rounding error correction factors used when converting float intermediates to
   // the underlying int-based pixels, etc.
   #define Half_8Bit  (0.5/256.0)
   #define Half_10Bit (0.5/1024.0)
   #define Half_16Bit (0.5/65536.0)

   // ITU-R BT.601 and 709 Constants and Coefficients
   // ===============================================
   // BT-601 (SD)
   #define Nr_BT601 (0.299)
   #define Ng_BT601 (0.587)
   #define Nb_BT601 (0.114)
   #define Kr_BT601 (0.5/0.701)
   #define Kb_BT601 (0.5/0.886)

   // scaling coefficients for Cr and Cb
   #define Cr0_601 ( Kr_BT601 - ( Kr_BT601 * Nr_BT601 ) )
   #define Cr1_601 ( -Kr_BT601 * Ng_BT601 )
   #define Cr2_601 ( -Kr_BT601 * Nb_BT601 )

   #define Cb0_601 ( - Kb_BT601 * Nr_BT601 )
   #define Cb1_601 ( - Kb_BT601 * Ng_BT601)
   #define Cb2_601 ( Kb_BT601 - ( Kb_BT601 * Nb_BT601 ) )

   // BT-709 (HD)
   #define Nr_BT709 (0.2126)
   #define Ng_BT709 (0.7152)
   #define Nb_BT709 (0.0722)
   #define Kr_BT709 (0.635)
   #define Kb_BT709 (0.539)

   // scaling coefficients for Cr and Cb
   #define Cr0_709 ( Kr_BT709 - ( Kr_BT709 * Nr_BT709 ) )
   #define Cr1_709 ( -Kr_BT709 * Ng_BT709 )
   #define Cr2_709 ( -Kr_BT709 * Nb_BT709 )

   #define Cb0_709 ( - Kb_BT709 * Nr_BT709 )
   #define Cb1_709 ( - Kb_BT709 * Ng_BT709)
   #define Cb2_709 ( Kb_BT709 - ( Kb_BT709 * Nb_BT709 ) )

   #define R0_601	( 1.0 / Kr_BT601 )
   #define G0_601	( Nr_BT601 / Kr_BT601 / Ng_BT601 )
   #define G1_601	( Nb_BT601 / Kb_BT601 / Ng_BT601 )
   #define B0_601	( 1.0 / Kb_BT601 )

   /*
      Scaling definitions, in float land:
      -----------------------------------

      RGB_Unscaled is defined as RGB[0..1]
      RGB_Computer is the same as RGB_Unscaled
      RGB_Video is defined as RGB[64/1023..940/1023](range 876)
      YCrCb_Unscaled is defined as Y[0..1] CrCb[-0.5..+0.5]
      YCrCb_Video is defined as Y[64/1023..940/1023](range 876) CrCb[64/1023..64/963](range 899). This suits BT.601 and BT.709
   */

   #define Scale_876_To_Norm  (1023.0/876.0)
   #define Scale_Norm_To_876  (876.0/1023.0)

   #define Scale_899_To_Norm  (1023.0/899.0)
   #define Scale_Norm_To_899  (899.0/1023.0)

   #define Scale_899_To_876   (876.0/899.0)
   #define Scale_876_To_899   (899.0/876.0)


   //not used
   //#define Scale_235_To_240   (963.0/940.0)
   //#define Scale_240_To_Norm  (1023.0/963.0)
   //#define Scale_Norm_To_240  (963.0/1023.0)

   #define Offset_64          (64.0/1024.0)
   #define Offset_512         (512.0/1024.0)

   #define DeOffset_64        (-64.0/1023.0)
   #define DeOffset_512       (-512.0/1023.0)

   //not used
   //#define AdjustOffset_16_From_899_To_876 ( ( 64.0 / 1023.0 ) * ( 5.0 / 899.0 ) )

   // Matrix Coefficients
   // ===================
   // Typically used as the constant definition of a float3x3
   //
   // Use like this:
   // float3 SourceValue = { ... }
   // float3 DestValue = SourceValue * Coefs;

   // Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT601
   // ------------------------------------------
   #define Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT601_MulFirst {                            \
      Nr_BT601,   Cr0_601,	Cb0_601,		\
      Ng_BT601,   Cr1_601,	Cb1_601,		\
      Nb_BT601,   Cr2_601,	Cb2_601 }

   // As above, but producing only the Y result (useful for sub-sampled schemes)
   #define Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT601_MulFirst_Y {                          \
      Nr_BT601,   \
      Ng_BT601,   \
      Nb_BT601 }

   #define Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT601_AddSecond { \
      0, Offset_512, Offset_512 }

   #define Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT601_AddSecond_Y \
      0

   // Coefs_RGB_Unscaled_To_YCrCb_Video_BT601
   // ---------------------------------------
   // Converts unscaled RGB values to video-scaled YCrCb values according to BT.601. Multiply
   // by _Mul coefs, then add the _Add coefs.
   // Nick:
   #define Coefs_RGB_Unscaled_To_YCrCb_Video_BT601_MulFirst {                                                                                              \
      Scale_Norm_To_876 * Nr_BT601,		Scale_Norm_To_899 * Cr0_601,		Scale_Norm_To_899 * Cb0_601,                   \
      Scale_Norm_To_876 * Ng_BT601,		Scale_Norm_To_899 * Cr1_601,        Scale_Norm_To_899 * Cb1_601,                   \
      Scale_Norm_To_876 * Nb_BT601,		Scale_Norm_To_899 * Cr2_601,        Scale_Norm_To_899 * Cb2_601	}

   // Tom:
   //#define Coefs_RGB_Unscaled_To_YCrCb_Video_BT601_MulFirst {                                                                                              \
   //   Scale_Norm_To_876 * Nr_BT601,   Scale_240_To_Norm * ( Scale_Norm_To_899 * Kr_BT601 - ( Kr_BT601 * Nr_BT601 ) ), ( Scale_Norm_To_899 * (-Kb_BT601) * Nr_BT601 ),                   \
   //   Scale_Norm_To_876 * Ng_BT601,   ( Scale_Norm_To_899 * (-Kr_BT601) * Ng_BT601 ),               ( Scale_Norm_To_899 * (-Kb_BT601) * Ng_BT601 ),                   \
   //   Scale_Norm_To_876 * Nb_BT601,   ( Scale_Norm_To_899 * (-Kr_BT601) * Nb_BT601 ),               ( Scale_Norm_To_899 * Kb_BT601 - ( Kb_BT601 * Nb_BT601 ) ) }

   // As above, but producing only the Y result (useful for sub-sampled schemes)
   #define Coefs_RGB_Unscaled_To_YCrCb_Video_BT601_MulFirst_Y { \
      Scale_Norm_To_876 * Nr_BT601,           \
      Scale_Norm_To_876 * Ng_BT601,           \
      Scale_Norm_To_876 * Nb_BT601 }

   #define Coefs_RGB_Unscaled_To_YCrCb_Video_BT601_AddSecond { \
      Offset_64, Offset_512, Offset_512 }

   // As above, but producing only the Y result (useful for sub-sampled schemes)
   #define Coefs_RGB_Unscaled_To_YCrCb_Video_BT601_AddSecond_Y \
      Offset_64

   // Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT709
   // ------------------------------------------
   // Converts unscaled RGB values to video-scaled YCrCb values according to BT.709. Multiply
   // by _Mul coefs, then add the _Add coefs.
   #define Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT709_MulFirst {                                   \
      Nr_BT709,   Cr0_709,	Cb0_709,		\
      Ng_BT709,   Cr1_709,	Cb1_709,		\
      Nb_BT709,   Cr2_709,	Cb2_709 }

   // As above, but producing only the Y result (useful for sub-sampled schemes)
   #define Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT709_MulFirst_Y { \
      Nr_BT709,           \
      Ng_BT709,           \
      Nb_BT709 }

   #define Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT709_AddSecond { \
      0, Offset_512, Offset_512 }

   #define Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT709_AddSecond_Y \
      0

   // Coefs_RGB_Unscaled_To_YCrCb_Video_BT709
   // ---------------------------------------
   // Converts unscaled RGB values to video-scaled YCrCb values according to BT.709. Multiply
   // by _Mul coefs, then add the _Add coefs.
   #define Coefs_RGB_Unscaled_To_YCrCb_Video_BT709_MulFirst {                                                                                              \
      Scale_Norm_To_876 * Nr_BT709,		Scale_Norm_To_899 * Cr0_709,		Scale_Norm_To_899 * Cb0_709,                   \
      Scale_Norm_To_876 * Ng_BT709,		Scale_Norm_To_899 * Cr1_709,        Scale_Norm_To_899 * Cb1_709,                   \
      Scale_Norm_To_876 * Nb_BT709,		Scale_Norm_To_899 * Cr2_709,        Scale_Norm_To_899 * Cb2_709	}

   // As above, but producing only the Y result (useful for sub-sampled schemes)
   #define Coefs_RGB_Unscaled_To_YCrCb_Video_BT709_MulFirst_Y { \
      Scale_Norm_To_876 * Nr_BT709,           \
      Scale_Norm_To_876 * Ng_BT709,           \
      Scale_Norm_To_876 * Nb_BT709 }

   #define Coefs_RGB_Unscaled_To_YCrCb_Video_BT709_AddSecond { \
      Offset_64, Offset_512, Offset_512 }

   // As above, but producing only the Y result (useful for sub-sampled schemes)
   #define Coefs_RGB_Unscaled_To_YCrCb_Video_BT709_AddSecond_Y \
      Offset_64

   // Coefs_YCrCb_Unscaled_BT601_To_RGB_Unscaled
   // ------------------------------------------
   #define Coefs_YCrCb_Unscaled_BT601_To_RGB_Unscaled_AddFirst {                \
      0, DeOffset_512, DeOffset_512 }

   #define Coefs_YCrCb_Unscaled_BT601_To_RGB_Unscaled_MulSecond {           \
                 1.0,			 1.0,       1.0,							\
		         R0_601,	- G0_601,       0.0,							\
                 0.0,		- G1_601,		B0_601}

   // Coefs_YCrCb_BT709_Unscaled_To_RGB_Unscaled
   // -----------------------------------------
   #define Coefs_YCrCb_Unscaled_BT709_To_RGB_Unscaled_AddFirst {                \
      0, DeOffset_512, DeOffset_512 }

   #define Coefs_YCrCb_Unscaled_BT709_To_RGB_Unscaled_MulSecond {           \
                 1.0,                              1.0,         1.0,       \
      1.0 / Kr_BT709, - Nr_BT709 / Kr_BT709 / Ng_BT709,         0.0,       \
                 0.0, - Nb_BT709 / Kb_BT709 / Ng_BT709, 1 / Kb_BT709 }

   // Coefs_YCrCb_Video_BT601_To_RGB_Unscaled
   // ---------------------------------------
   // Convert video-scaled YCrCb to RGB unscaled. Add the _Add factors to the YCrCb val, then mul by _Mul.
   #define Coefs_YCrCb_Video_BT601_To_RGB_Unscaled_AddFirst {                \
      DeOffset_64, DeOffset_512, DeOffset_512 }

   #define Coefs_YCrCb_Video_BT601_To_RGB_Unscaled_MulSecond {											\
		Scale_876_To_Norm,				 Scale_876_To_Norm,					Scale_876_To_Norm,			\
		Scale_899_To_Norm * R0_601,		-Scale_899_To_Norm * G0_601,        0.0,						\
		0.0,							-Scale_899_To_Norm * G1_601,		Scale_899_To_Norm * B0_601 }

   // Coefs_YCrCb_Video_BT709_To_RGB_Unscaled
   // ---------------------------------------
   #define Coefs_YCrCb_Video_BT709_To_RGB_Unscaled_AddFirst {                \
      DeOffset_64, DeOffset_512, DeOffset_512 }

   #define Coefs_YCrCb_Video_BT709_To_RGB_Unscaled_MulSecond {                \
                 Scale_876_To_Norm,                                   Scale_876_To_Norm,            Scale_876_To_Norm,       \
      Scale_899_To_Norm / Kr_BT709, -Scale_899_To_Norm * Nr_BT709 / Kr_BT709 / Ng_BT709,                          0.0,       \
                               0.0, -Scale_899_To_Norm * Nb_BT709 / Kb_BT709 / Ng_BT709, Scale_899_To_Norm / Kb_BT709 }

   // Coefs_YCrCb_Video_BT601_To_RGB_Video
   // ---------------------------------------
   // Convert video-scaled YCrCb to RGB video. Add the _Add factors to the YCrCb val, then mul by _Mul.
   #define Coefs_YCrCb_Video_BT601_To_RGB_Video_AddFirst {                \
      DeOffset_64, DeOffset_512, DeOffset_512 }

   #define Coefs_YCrCb_Video_BT601_To_RGB_Video_MulSecond {											\
        1.0,                            1.0,							1.0,						\
		Scale_899_To_876 * R0_601,		-Scale_899_To_876 * G0_601,     0.0,						\
        0.0,							-Scale_899_To_876 * G1_601,		Scale_899_To_876 * B0_601 }

   #define Coefs_YCrCb_Video_BT601_To_RGB_Video_AddThird {                \
      Offset_64, Offset_64, Offset_64 }

   // Coefs_YCrCb_Unscaled_BT601_To_RGB_Video
   // ---------------------------------------
   // Convert unscaled YCrCb to RGB video. Add the _Add factors to the YCrCb val, then mul by _Mul.
   #define Coefs_YCrCb_Unscaled_BT601_To_RGB_Video_AddFirst {                \
      0, DeOffset_512, DeOffset_512 }

   #define Coefs_YCrCb_Unscaled_BT601_To_RGB_Video_MulSecond {										\
			1.0,                        1.0,							1.0,						\
			Scale_Norm_To_876 * R0_601, -Scale_Norm_To_876 * G0_601,    0.0,						\
            0.0,						-Scale_Norm_To_876 * G1_601,	Scale_Norm_To_876 * B0_601 }

   #define Coefs_YCrCb_Unscaled_BT601_To_RGB_Video_AddThird {                \
      Offset_64, Offset_64, Offset_64 }

   // Coefs_YCrCb_Video_BT709_To_RGB_Video
   // ---------------------------------------
   // Convert video-scaled YCrCb to RGB video. Add the _Add factors to the YCrCb val, then mul by _Mul.
   #define Coefs_YCrCb_Video_BT709_To_RGB_Video_AddFirst {                \
      DeOffset_64, DeOffset_512, DeOffset_512 }

   #define Coefs_YCrCb_Video_BT709_To_RGB_Video_MulSecond {                \
                              1.0,                                                1.0,                         1.0,       \
      Scale_899_To_876 / Kr_BT709, -Scale_899_To_876 * Nr_BT709 / Kr_BT709 / Ng_BT709,                         0.0,       \
                              0.0, -Scale_899_To_876 * Nb_BT709 / Kb_BT709 / Ng_BT709, Scale_899_To_876 / Kb_BT709 }

   #define Coefs_YCrCb_Video_BT709_To_RGB_Video_AddThird {                \
      Offset_64, Offset_64, Offset_64 }

   // Coefs_YCrCb_Unscaled_BT709_To_RGB_Video
   // ---------------------------------------
   // Convert unscaled YCrCb to RGB video. Add the _Add factors to the YCrCb val, then mul by _Mul.
   #define Coefs_YCrCb_Unscaled_BT709_To_RGB_Video_AddFirst {                \
      0, DeOffset_512, DeOffset_512 }

   #define Coefs_YCrCb_Unscaled_BT709_To_RGB_Video_MulSecond {                \
                               1.0,                                                 1.0,                          1.0,       \
      Scale_Norm_To_876 / Kr_BT709, -Scale_Norm_To_876 * Nr_BT709 / Kr_BT709 / Ng_BT709,                          0.0,       \
                               0.0, -Scale_Norm_To_876 * Nb_BT709 / Kb_BT709 / Ng_BT709, Scale_Norm_To_876 / Kb_BT709 }

   #define Coefs_YCrCb_Unscaled_BT709_To_RGB_Video_AddThird {                \
      Offset_64, Offset_64, Offset_64 }

   struct VSO_1Tex
   {
      float2 TexUV0 : TEXCOORD1;
      float2 TexUV1 : TEXCOORD2;
   };

   struct VSO_2Tex
   {
      float2 TexUV0 : TEXCOORD1;
      float2 TexUV1 : TEXCOORD2;
   };

   struct VSO_5Tex
   {
      float2 TexUV0 : TEXCOORD1;
      float2 TexUV1 : TEXCOORD2;
      float2 TexUV2 : TEXCOORD3;
      float2 TexUV3 : TEXCOORD4;
      float2 TexUV4 : TEXCOORD5;
   };

   struct VSO_6Tex
   {
      float2 TexUV0 : TEXCOORD1;
      float2 TexUV1 : TEXCOORD2;
      float2 TexUV2 : TEXCOORD3;
      float2 TexUV3 : TEXCOORD4;
      float2 TexUV4 : TEXCOORD5;
      float2 TexUV5 : TEXCOORD6;
   };

   struct VSO_8Tex
   {
      float2 TexUV0 : TEXCOORD0;
      float2 TexUV1 : TEXCOORD1;
      float2 TexUV2 : TEXCOORD2;
      float2 TexUV3 : TEXCOORD3;
      float2 TexUV4 : TEXCOORD4;
      float2 TexUV5 : TEXCOORD5;
      float2 TexUV6 : TEXCOORD6;
      float2 TexUV7 : TEXCOORD7;
   };

   texture  gSourceImage;			      // The source image
   float    gSourceWidthPixelsInv;     // 1/<width of source image in pixels>
   float    gSourceWidthPixelsInvD2;   // 0.5/<width of source image in pixels>

   texture  gIterHelp2Step;            // A 2x1 texture comprising one black (0,0,0,0) pixel
                                       // followed by one white (1,1,1,1) pixel.

   //--------------------------------------------------------------------------------------
   // Texture samplers
   //--------------------------------------------------------------------------------------
   sampler SourceImageSampler =
   sampler_state
   {
      Texture = <gSourceImage>;
      MipFilter = POINT;
      MinFilter = POINT;
      MagFilter = POINT; //GAUSSIANQUAD; //POINT; //LINEAR;
      AddressU  = ClampToEdge;
      AddressV  = ClampToEdge;
   };

   sampler IterHelp2StepSampler =
   sampler_state
   {
      Texture = <gIterHelp2Step>;
      MipFilter = POINT;
      MinFilter = POINT;
      MagFilter = POINT;
      AddressU  = WRAP;
      AddressV  = WRAP;
   };

   int isEven( int i )
   {
     return ( ( 2 * ( i / 2 ) ) == i );
   }

   //-------------------------------------------------------------------------------------------------
   /* Nomenclature notes:

      RGB   - Implicltly 4:4:4 sampling unless stated otherwise
      YCrCb - Always follow with sampling ratio (422, 444 etc) as there is no clear leader.
   */
   // CCS_SELECT_FUNCTION - Must be one of:
   #define CCS_SELECT_FUNCTION_RGB_To_YCrCb422        1
   #define CCS_SELECT_FUNCTION_RGB_To_YCrCb444        2
   #define CCS_SELECT_FUNCTION_YCrCb422_To_RGB        3
   //-------------------------------------------------------------------------------------------------
   // Scaling Constants:
   #define CCS_SCALING_UNSCALED 1
   #define CCS_SCALING_COMPUTER CCS_SCALING_UNSCALED  // synonym
   #define CCS_SCALING_VIDEO    2
   // CCS_RGB_SCALING - Must be one of the above scaling constants
   // CCS_YUV_SCALING - Must be one of the above scaling constants
   //-------------------------------------------------------------------------------------------------
   // CCS_YUV_COLOURSPACE - Must be one of:
   #define CCS_YUV_COLOURSPACE_BT601      1
   #define CCS_YUV_COLOURSPACE_BT709      2
   //-------------------------------------------------------------------------------------------------
   // Some filters have different implementations for the edges of an image. The branches choice is
   // based on the output pixels, and multiple quads with different techniques must be drawn to suit.
   // CCS_FILTER_BRANCH - Must be one of:
   #define CCS_FILTER_BRANCH_COL_1
   #define CCS_FILTER_BRANCH_COL_2
   //-------------------------------------------------------------------------------------------------
   // CCS_FILTER_TYPE - Must be one of:
   #define CCS_FILTER_TYPE_POINT          1
   #define CCS_FILTER_TYPE_CATMULL_ROM    2
   #define CCS_FILTER_TYPE_IDEAL_MITCHELL 3
   #define CCS_FILTER_TYPE_LANCZOS        4
   // NB: Exact meaning and texture tap requirements vary.

#endif

//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------- CCS_FUNCTIONS --
//-------------------------------------------------------------------------------------------------
// Define CCS_FUNCTIONS to enable including of function code.
//
// CCS_SELECT_FUNCTION must be defined as detailed above to select which function to compile.
// CCS_FUNCTION_NAME must be defined as the name you want for the compile function.
//
// Further symbols must be defined to provide the parameters that function requires.

#ifdef CCS_FUNCTIONS
   //-------------------------------------------------------------------------------------------------
   //---------------------------------------------------------- CCS_SELECT_FUNCTION_RGB_To_YCrCb422 --
   //---------------------------------------------------------- CCS_SELECT_FUNCTION_RGB_To_YCrCb444 --
   //-------------------------------------------------------------------------------------------------
   // These two functions have all the coeficient selection in common, so they are grouped here

   // Provide additional definitions as follows:
   // CCS_RGB_SCALING
   // CCS_YUV_SCALING
   // CCS_YUV_COLOURSPACE
   // CCS_FILTER_TYPE
   // Vertex definitions:
   // TexUV0 - Screen-aligned quad (with texel centering correction)
   // TexUV1 - As UV0, but TexUV0.x + one source pixel width;
   // If using 422 with Lanczos filter, see notes below

   #if ( CCS_SELECT_FUNCTION == CCS_SELECT_FUNCTION_RGB_To_YCrCb422 ) || ( CCS_SELECT_FUNCTION == CCS_SELECT_FUNCTION_RGB_To_YCrCb444 )

      #if ( CCS_SELECT_FUNCTION == CCS_SELECT_FUNCTION_RGB_To_YCrCb422 )

         #if ( CCS_FILTER_TYPE == CCS_FILTER_TYPE_POINT )

            #define CCSI_IN_TYPE VSO_2Tex

         #endif

         #if ( CCS_FILTER_TYPE == CCS_FILTER_TYPE_LANCZOS )

            #define CCSI_IN_TYPE VSO_6Tex

         #endif

      #endif

      #if ( CCS_SELECT_FUNCTION == CCS_SELECT_FUNCTION_RGB_To_YCrCb444 )

         #define CCSI_IN_TYPE VSO_1Tex

      #endif

         float4 CCS_FUNCTION_NAME ( CCSI_IN_TYPE In ) : COLOR0
         {
            // Coeficient selection branches common to this group of functions
            #if ( CCS_YUV_COLOURSPACE == CCS_YUV_COLOURSPACE_BT601 )

               #if ( CCS_RGB_SCALING == CCS_SCALING_UNSCALED )

                  #if ( CCS_YUV_SCALING == CCS_SCALING_VIDEO )

                     const float3x3 coefsMul  = Coefs_RGB_Unscaled_To_YCrCb_Video_BT601_MulFirst;
                     const float3   coefsAdd  = Coefs_RGB_Unscaled_To_YCrCb_Video_BT601_AddSecond;
                     const float3   coefsMulY = Coefs_RGB_Unscaled_To_YCrCb_Video_BT601_MulFirst_Y;
                     const float    coefsAddY = Coefs_RGB_Unscaled_To_YCrCb_Video_BT601_AddSecond_Y;

                  #endif

                  #if ( CCS_YUV_SCALING == CCS_SCALING_UNSCALED )

                     const float3x3 coefsMul  = Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT601_MulFirst;
                     const float3   coefsAdd  = Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT601_AddSecond;
                     const float3   coefsMulY = Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT601_MulFirst_Y;
                     const float    coefsAddY = Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT601_AddSecond_Y;

                  #endif

               #endif

               #if ( CCS_RGB_SCALING == CCS_SCALING_VIDEO )

                  #if ( CCS_YUV_SCALING == CCS_SCALING_VIDEO )

                     const float3x3 coefsMul  = Coefs_RGB_Video_To_YCrCb_Video_BT601_MulFirst;
                     const float3   coefsAdd  = Coefs_RGB_Video_To_YCrCb_Video_BT601_AddSecond;
                     const float3   coefsMulY = Coefs_RGB_Video_To_YCrCb_Video_BT601_MulFirst_Y;
                     const float    coefsAddY = Coefs_RGB_Video_To_YCrCb_Video_BT601_AddSecond_Y;

                  #endif

                  #if ( CCS_YUV_SCALING == CCS_SCALING_UNSCALED )

                     const float3x3 coefsMul  = Coefs_RGB_Video_To_YCrCb_Unscaled_BT601_MulFirst;
                     const float3   coefsAdd  = Coefs_RGB_Video_To_YCrCb_Unscaled_BT601_AddSecond;
                     const float3   coefsMulY = Coefs_RGB_Video_To_YCrCb_Unscaled_BT601_MulFirst_Y;
                     const float    coefsAddY = Coefs_RGB_Video_To_YCrCb_Unscaled_BT601_AddSecond_Y;

                  #endif

               #endif

            #endif // CCS_YUV_COLOURSPACE == CCS_YUV_COLOURSPACE_BT601

            #if ( CCS_YUV_COLOURSPACE == CCS_YUV_COLOURSPACE_BT709 )

               #if ( CCS_RGB_SCALING == CCS_SCALING_UNSCALED )

                  #if ( CCS_YUV_SCALING == CCS_SCALING_VIDEO )

                     const float3x3 coefsMul  = Coefs_RGB_Unscaled_To_YCrCb_Video_BT709_MulFirst;
                     const float3   coefsAdd  = Coefs_RGB_Unscaled_To_YCrCb_Video_BT709_AddSecond;
                     const float3   coefsMulY = Coefs_RGB_Unscaled_To_YCrCb_Video_BT709_MulFirst_Y;
                     const float    coefsAddY = Coefs_RGB_Unscaled_To_YCrCb_Video_BT709_AddSecond_Y;

                  #endif

                  #if ( CCS_YUV_SCALING == CCS_SCALING_UNSCALED )

                     const float3x3 coefsMul  = Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT709_MulFirst;
                     const float3   coefsAdd  = Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT709_AddSecond;
                     const float3   coefsMulY = Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT709_MulFirst_Y;
                     const float    coefsAddY = Coefs_RGB_Unscaled_To_YCrCb_Unscaled_BT709_AddSecond_Y;

                  #endif

               #endif

               #if ( CCS_RGB_SCALING == CCS_SCALING_VIDEO )

                  #if ( CCS_YUV_SCALING == CCS_SCALING_VIDEO )

                     const float3x3 coefsMul  = Coefs_RGB_Video_To_YCrCb_Video_BT709_MulFirst;
                     const float3   coefsAdd  = Coefs_RGB_Video_To_YCrCb_Video_BT709_AddSecond;
                     const float3   coefsMulY = Coefs_RGB_Video_To_YCrCb_Video_BT709_MulFirst_Y;
                     const float    coefsAddY = Coefs_RGB_Video_To_YCrCb_Video_BT709_AddSecond_Y;

                  #endif

                  #if ( CCS_YUV_SCALING == CCS_SCALING_UNSCALED )

                     const float3x3 coefsMul  = Coefs_RGB_Video_To_YCrCb_Unscaled_BT709_MulFirst;
                     const float3   coefsAdd  = Coefs_RGB_Video_To_YCrCb_Unscaled_BT709_AddSecond;
                     const float3   coefsMulY = Coefs_RGB_Video_To_YCrCb_Unscaled_BT709_MulFirst_Y;
                     const float    coefsAddY = Coefs_RGB_Video_To_YCrCb_Unscaled_BT709_AddSecond_Y;

                  #endif

               #endif

            #endif // CCS_YUV_COLOURSPACE == CCS_YUV_COLOURSPACE_BT709

            #if ( CCS_SELECT_FUNCTION == CCS_SELECT_FUNCTION_RGB_To_YCrCb422 )

               //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
               #if ( CCS_FILTER_TYPE == CCS_FILTER_TYPE_POINT )

                  // U and V (Cr and Cb) samples are co-sited with odd-numbered Y-samples (eg, 1st, 3rd, 5th)

                  float4 ret;

                  float4 inDiffuse1 = tex2D( SourceImageSampler, In.TexUV0 );
                  float4 inDiffuse2 = tex2D( SourceImageSampler, In.TexUV1 );

                  // First pixel (provides Y, Cr and Cb samples at this position)
                  ret.bag = mul( inDiffuse1.rgb, coefsMul ) + coefsAdd;

                  // Second pixel (Provides Y-sample at this position only)
                  ret.r = mul( inDiffuse2.rgb, coefsMulY ) + coefsAddY;

                  // Rounding correction
                  ret += Half_10Bit;

                  return ret;

               #endif // CCS_FILTER_TYPE == CCS_FILTER_TYPE_POINT
               //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
               #if ( CCS_FILTER_TYPE == CCS_FILTER_TYPE_LANCZOS )
            /*
                  for(;x<pic_w-3;x+=2){
                  tmp[x>>1]=OC_CLAMPI(0,3*(_aux[x-2]+_aux[x+3])-17*(_aux[x-1]+_aux[x+2])+
                  78*(_aux[x]+_aux[x+1])+64>>7,255);
            */

                  // HACK: to get it going...
                  // We pretend we're in the middle of the image for testing. The image will be
                  // shifted left a bit, and the right edge will be wrong.
                  float4 ret;

                  float3 xMinus2 = mul( tex2D( SourceImageSampler, In.TexUV0 ).rgb, coefsMul ) + coefsAdd;
                  float3 xMinus1 = mul( tex2D( SourceImageSampler, In.TexUV1 ).rgb, coefsMul ) + coefsAdd;
                  float3 x       = mul( tex2D( SourceImageSampler, In.TexUV2 ).rgb, coefsMul ) + coefsAdd;
                  float3 xPlus1  = mul( tex2D( SourceImageSampler, In.TexUV3 ).rgb, coefsMul ) + coefsAdd;
                  float3 xPlus2  = mul( tex2D( SourceImageSampler, In.TexUV4 ).rgb, coefsMul ) + coefsAdd;
                  float3 xPlus3  = mul( tex2D( SourceImageSampler, In.TexUV5 ).rgb, coefsMul ) + coefsAdd;
                  //float3 xPlus4  = mul( tex2D( SourceImageSampler, In.TexUV6 ).rgb, coefsMul ) + coefsAdd;

                  // Y0
                  ret.b = x.r;

                  // U
                  ret.a = ( ( 3.0 * 255.0 ) * ( xMinus2.g + xPlus3.g ) - ( 17.0 * 255.0 ) * (xMinus1.g + xPlus2.g ) +
                        ( 78.0 * 255.0 ) * ( x.g + xPlus1.g ) + 64.0 ) / ( 128.0 * 255.0 );
                  //ret.a = ( ( 3.0 / 255.0 ) * ( xMinus2.g + xPlus3.g ) - ( 17.0 / 255.0 ) * (xMinus1.g + xPlus2.g ) +
                  //        ( 78.0 / 255.0 ) * ( x.g + xPlus1.g ) + ( 64.0 / 255.0 ) ) / ( 7.0 / 255.0 );
                  // Y1
                  ret.r = xPlus1.r;

                  // V
                  ret.g = ( ( 3.0 * 255.0 ) * ( xMinus2.b + xPlus3.b ) - ( 17.0 * 255.0 ) * (xMinus1.b + xPlus2.b ) +
                        ( 78.0 * 255.0 ) * ( x.b + xPlus1.b ) + 64.0 ) / ( 128.0 * 255.0 );

                  // First pixel (provides Y, Cr and Cb samples at this position)
                  //ret.bag = mul( inDiffuse1.rgb, coefsMul ) + coefsAdd;

                  // Second pixel (Provides Y-sample at this position only)
                  //ret.r = mul( inDiffuse2.rgb, coefsMulY ) + coefsAddY;

                  // Rounding correction
                  ret += Half_10Bit;

                  return ret;

               #endif // CCS_FILTER_TYPE == CCS_FILTER_TYPE_LANCZOS

            #endif // CCS_SELECT_FUNCTION == CCS_SELECT_FUNCTION_RGB_To_YCrCb422

            #if ( CCS_SELECT_FUNCTION == CCS_SELECT_FUNCTION_RGB_To_YCrCb444 )

                  // U and V (Cr and Cb) samples are co-sited with every Y-sample

                  float4 ret;

                  ret.a = 1;

                  ret.rgb = mul( tex2D( SourceImageSampler, In.TexUV0 ).rgb, coefsMul ) + coefsAdd;

                  // Rounding correction
                  ret.rgb += Half_10Bit;

                  return ret;

            #endif // CCS_SELECT_FUNCTION == CCS_SELECT_FUNCTION_RGB_To_YCrCb444
         }

   #endif   // ( CCS_SELECT_FUNCTION == CCS_SELECT_FUNCTION_RGB_To_YCrCb422 ) ||
            // ( CCS_SELECT_FUNCTION == CCS_SELECT_FUNCTION_RGB_To_YCrCb444 )

   //-------------------------------------------------------------------------------------------------
   //---------------------------------------------------- CCS_SELECT_FUNCTION_YCrCb422_To_RGB --
   //-------------------------------------------------------------------------------------------------
   // Upsample YUV422 data to 444 with choice of filtering, convert to RGB
   #if ( CCS_SELECT_FUNCTION == CCS_SELECT_FUNCTION_YCrCb422_To_RGB )

      #if ( CCS_FILTER_TYPE == CCS_FILTER_TYPE_POINT )

         // TODO: Put q in tex1 (currently always put in and read from tex4)
         #define CCSI_IN_TYPE VSO_5Tex

      #else

         #define CCSI_IN_TYPE VSO_5Tex

      #endif

      float4 CCS_FUNCTION_NAME ( CCSI_IN_TYPE In ) : COLOR0
      {
         #if ( CCS_YUV_COLOURSPACE == CCS_YUV_COLOURSPACE_BT601 )

            #if ( CCS_RGB_SCALING == CCS_SCALING_UNSCALED )

               #if ( CCS_YUV_SCALING == CCS_SCALING_VIDEO )

                  const float3   ccCoefsAdd1  = Coefs_YCrCb_Video_BT601_To_RGB_Unscaled_AddFirst;
                  const float3x3 ccCoefsMul2  = Coefs_YCrCb_Video_BT601_To_RGB_Unscaled_MulSecond;

               #endif

               #if ( CCS_YUV_SCALING == CCS_SCALING_UNSCALED )

                  const float3   ccCoefsAdd1  = Coefs_YCrCb_Unscaled_BT601_To_RGB_Unscaled_AddFirst;
                  const float3x3 ccCoefsMul2  = Coefs_YCrCb_Unscaled_BT601_To_RGB_Unscaled_MulSecond;

               #endif

            #endif

            #if ( CCS_RGB_SCALING == CCS_SCALING_VIDEO )

               #if ( CCS_YUV_SCALING == CCS_SCALING_VIDEO )

                  const float3   ccCoefsAdd1  = Coefs_YCrCb_Video_BT601_To_RGB_Video_AddFirst;
                  const float3x3 ccCoefsMul2  = Coefs_YCrCb_Video_BT601_To_RGB_Video_MulSecond;
                  const float3   ccCoefsAdd3  = Coefs_YCrCb_Video_BT601_To_RGB_Video_AddThird;

               #endif

               #if ( CCS_YUV_SCALING == CCS_SCALING_UNSCALED )

                  const float3   ccCoefsAdd1  = Coefs_YCrCb_Unscaled_BT601_To_RGB_Video_AddFirst;
                  const float3x3 ccCoefsMul2  = Coefs_YCrCb_Unscaled_BT601_To_RGB_Video_MulSecond;
                  const float3   ccCoefsAdd3  = Coefs_YCrCb_Unscaled_BT601_To_RGB_Video_AddThird;

               #endif

            #endif

         #endif // CCS_YUV_COLOURSPACE == CCS_YUV_COLOURSPACE_BT601

         #if ( CCS_YUV_COLOURSPACE == CCS_YUV_COLOURSPACE_BT709 )

            #if ( CCS_RGB_SCALING == CCS_SCALING_UNSCALED )

               #if ( CCS_YUV_SCALING == CCS_SCALING_VIDEO )

                  const float3   ccCoefsAdd1  = Coefs_YCrCb_Video_BT709_To_RGB_Unscaled_AddFirst;
                  const float3x3 ccCoefsMul2  = Coefs_YCrCb_Video_BT709_To_RGB_Unscaled_MulSecond;

               #endif

               #if ( CCS_YUV_SCALING == CCS_SCALING_UNSCALED )

                  const float3   ccCoefsAdd1  = Coefs_YCrCb_Unscaled_BT709_To_RGB_Unscaled_AddFirst;
                  const float3x3 ccCoefsMul2  = Coefs_YCrCb_Unscaled_BT709_To_RGB_Unscaled_MulSecond;

               #endif

            #endif

            #if ( CCS_RGB_SCALING == CCS_SCALING_VIDEO )

               #if ( CCS_YUV_SCALING == CCS_SCALING_VIDEO )

                  const float3   ccCoefsAdd1  = Coefs_YCrCb_Video_BT709_To_RGB_Video_AddFirst;
                  const float3x3 ccCoefsMul2  = Coefs_YCrCb_Video_BT709_To_RGB_Video_MulSecond;
                  const float3   ccCoefsAdd3  = Coefs_YCrCb_Video_BT709_To_RGB_Video_AddThird;

               #endif

               #if ( CCS_YUV_SCALING == CCS_SCALING_UNSCALED )

                  const float3   ccCoefsAdd1  = Coefs_YCrCb_Unscaled_BT709_To_RGB_Video_AddFirst;
                  const float3x3 ccCoefsMul2  = Coefs_YCrCb_Unscaled_BT709_To_RGB_Video_MulSecond;
                  const float3   ccCoefsAdd3  = Coefs_YCrCb_Unscaled_BT709_To_RGB_Video_AddThird;

               #endif

            #endif

         #endif // CCS_YUV_COLOURSPACE == CCS_YUV_COLOURSPACE_BT709

         // Source pixels are stored as:
         // Y0 = R, Cr = G, Y1 = A, Cb = B

         #if ( CCS_FILTER_TYPE == CCS_FILTER_TYPE_CATMULL_ROM )

            const float2x4 filt =
            {
               { 0,       1,      0,       0      },  // Catmull-Rom (b=0, c=0.5) @ x=0.0
               { -0.0625, 0.5625, 0.5625, -0.0625 }   // Catmull-Rom (b=0, c=0.5) @ x=0.5
            };

         #endif

         #if ( CCS_FILTER_TYPE == CCS_FILTER_TYPE_IDEAL_MITCHELL )

            const float2x4 filt =
            {
               {  0.0555556, 0.8888889, 0.0555556,  0.0       },  // Viewer test ideal, aka Mitchell (b=1/3, c=1/3) @ x=0.0
               { -0.0347222, 0.5347222, 0.5347222, -0.0347222 }   // Viewer test ideal, aka Mitchell (b=1/3, c=1/3) @ x=0.5
            };

         #endif

         // TOTALLY CORRECT INFO:
         // For each input ARGB pixel, Y0 is in pix[0], Y1 is in pix[2]
         float3 yuv;

         int q = isEven( ( int ) In.TexUV4.x );

         #if ( CCS_FILTER_TYPE == CCS_FILTER_TYPE_POINT )

            float4 src = tex2D( SourceImageSampler, In.TexUV1 );

            // Y
            yuv.r  = src[ q * 2 ];

            // UV
            yuv.gb = src.ag;

         #else

            float4x2 s;

            // Y
            float4 tex1 = tex2D( SourceImageSampler, In.TexUV1 );
            yuv.r = tex1[ q * 2 ];

            s[0]= tex2D( SourceImageSampler, In.TexUV0 ).ag;
            s[1]= tex1.ag;                                  // <--- on even output samples, we will sample here (x=0.0)
            //                                                 <--- on odd output samples, we will sample between these two pixels (x=0.5)
            s[2]= tex2D( SourceImageSampler, In.TexUV2 ).ag;
            s[3]= tex2D( SourceImageSampler, In.TexUV3 ).ag;

            // uv
            yuv.gb = mul( filt[1-q], s );

         #endif

         float4 ret;

         ret.a = 1;

         #if ( CCS_RGB_SCALING == CCS_SCALING_VIDEO )

            // Need to use additional 'add' operation
            ret.rgb = mul( yuv + ccCoefsAdd1, ccCoefsMul2 ) + ccCoefsAdd3;

         #endif

         #if ( CCS_RGB_SCALING == CCS_SCALING_UNSCALED )

            ret.rgb = mul( yuv + ccCoefsAdd1, ccCoefsMul2 );

         #endif

         // Rounding correction
         // sadly black to be 1,1,1 so this is a quick hack until I understand better
         //ret.rgb += Half_10Bit;

         return ret;
      }

   #endif // #if ( CCS_SELECT_FUNCTION == CCS_SELECT_FUNCTION_YCrCb422_To_RGB )

   // Now clean up by clearing all possible macros, forcing the includer to explictly set them
   // up for the next function.
   #undef CCS_FUNCTION_NAME
   #undef CCS_SELECT_FUNCTION
   #undef CCS_RGB_SCALING
   #undef CCS_YUV_SCALING
   #undef CCS_YUV_COLOURSPACE
   #undef CCS_FILTER_TYPE

   #undef CCSI_IN_TYPE

#endif // CCS_FUNCTIONS

/*
http://lists.xiph.org/pipermail/theora-dev/2004-September/002413.html

If one _is_ going to apply an alias-correcting filter to upsampling, one
can do better than the Catmull-Rom interpolation suggested in that
article. Mitchell and Netravali
(http://portal.acm.org/citation.cfm?id=378514) investigated an entire
family of bicubic interpolating filters:

k(x)=(1/6)*{(12-9B-6C)|x|^3+(-18+12B-6C)|x|^2+(6-2B),             |x|<1,
             (-B-6C)|x|^3+(6B+30C)|x|^2+(-12B-48C)|x|-(8B+24C), 1<=|x|<2,
             0                                                  2<=|x| }

The values B and C are parameters. (1,0) corresponds to the traditional
cubic spline. (0,C) is the 1-parameter family of splines which exactly
interpolate their sample locations ("cardinal cubics"), and (0,1/2)
corresponds to the Catmull-Rom spline. They performed tests with human
observers and found a region of the parameter space which provides a
good trade-off between blurring, ringing, and anisotropic artifacts,
centered around the values (1/3,1/3).

This yields the standard Mitchell filter:
k(x)=(1/6)*{7|x|^3-12|x|^2+16/3,               |x|<1
             -(7/3)|x|^3+12|x|^2-20|x|+32/3, 1<=|x|<2
             0                               2<=|x|}

This is what is used for upsampling in the experimental encoder_example,
at
http://svn.xiph.org/experimental/derf/theora-exp/examples/encoder_example.c
*/
