//--------------------------------------------------------------//
// Colour Correction 4
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "CC4";
   string Category    = "Colour";
> = 0;

//--------------------------------------------------------------
// Params
//--------------------------------------------------------------
float4 BlackLevel;
float4 GreyLevel;
float4 WhiteLevel;
float4 ShadowTintColour;
float4 MidTintColour;
float4 HighTintColour;
float  Saturation;
float4 Gamma;
float4 Contrast;
float4 Brightness;
float4 Gain;
float  HueBias;
float  SatBias;
float  ValBias;
float  SatScale;
float  ValScale;

float _pi     = 3.14159;
float _pi2    = 6.2831853;
float _offset = 0.5 / 256.0;
float _scale  = 255.0 / 256.0;
float _rt3    = 1.0 / sqrt( 3.0 );

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture Input;
texture _lut;
texture _hsl : RenderColorTarget;

sampler InputSampler = sampler_state { Texture = <Input>; };
sampler LUTSampler   = sampler_state { Texture = <_lut>; };
sampler HSLSampler   = sampler_state { Texture = <_hsl>; };

//---------------------------------------------------------------------------------------//
// Logic
//---------------------------------------------------------------------------------------//
float4 main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   // Read the source image pixel
   float4 src = tex2D( InputSampler, xy1 );
   float4 ret = src;

   // Do balancing
   float greyAverage = ( GreyLevel.r + GreyLevel.g + GreyLevel.b ) / 3.0;
   ret -= BlackLevel;
   ret /= ( WhiteLevel - BlackLevel );
   ret -= GreyLevel;
   ret += greyAverage;
   ret = clamp( ret, 0.0, 1.0 );

   // Calc its brightness (so that it can be treated as a shadow/midtone/highlight as appropriate)
   float lum = ( ret.r + ret.g + ret.b ) / 3.0;

   // Apply the shadow/midtone/highlight tints
   float2 lutPos   = float2( ( lum * _scale ) + _offset, 0.5 );
   float4 lutPixel = tex2D( LUTSampler, lutPos );
   ret += ShadowTintColour * lutPixel.r;
   ret += MidTintColour    * lutPixel.g;
   ret += HighTintColour   * lutPixel.b;

   float4 avg = ( ret.r + ret.g + ret.b ) / 3.0;
   ret = avg + ( ( ret - avg ) * Saturation );

   // Do brightness, gamma & contrast logic
   ret = ( ( ( ( pow( saturate( ret ), 1 / Gamma ) * Gain ) + Brightness ) - 0.5 ) * Contrast ) + 0.5;

   ret.a = src.a;

   return ret;
}

//--------------------------------------------------------------//
// HSLtoRGB
//--------------------------------------------------------------//
float4 RGBtoHSL( float2 xy1 : TEXCOORD1 ) : COLOR
{
   // Read the source image pixel
   float4 src = tex2D( InputSampler, xy1 );
   float4 ret = src;

   // Do balancing
   float greyAverage = ( GreyLevel.r + GreyLevel.g + GreyLevel.b ) / 3.0;
   ret -= BlackLevel;
   ret /= ( WhiteLevel - BlackLevel );
   ret -= GreyLevel;
   ret += greyAverage;
   ret = clamp( ret, 0.0, 1.0 );

   // Calc its brightness (so that it can be treated as a shadow/midtone/highlight as appropriate)
   float lum = ( ret.r + ret.g + ret.b ) / 3.0;

   // Apply the shadow/midtone/highlight tints
   float2 lutPos   = float2( ( lum * _scale ) + _offset, 0.5 );
   float4 lutPixel = tex2D( LUTSampler, lutPos );
   ret += ShadowTintColour * lutPixel.r;
   ret += MidTintColour    * lutPixel.g;
   ret += HighTintColour   * lutPixel.b;

   float4 avg = ( ret.r + ret.g + ret.b ) / 3.0;
   ret = avg + ( ( ret - avg ) * Saturation );

   // Do brightness, gamma & contrast logic
   ret = ( ( ( ( pow( saturate( ret ), 1 / Gamma ) * Gain ) + Brightness ) - 0.5 ) * Contrast ) + 0.5;

   float r  = saturate( ret.r );
   float g  = saturate( ret.g );
   float b  = saturate( ret.b );
   float rg = r - g;
   float rb = r - b;

   lum = ( r + g + b ) / 3.0;

   // Hue
   float h = acos( ( rg + rb ) / ( 2 * sqrt( rg * rg + rb * ( g - b ) ) ) ) / _pi2;

   if ( b > g )
      h = 1.0 - h;

   // Saturation
   float s = 1.0 - min( min( r, g ), b ) / lum;

   return float4( h, s, lum, src.a );
}

//--------------------------------------------------------------//
// HSLtoRGB
//--------------------------------------------------------------//
float4 HSLtoRGB( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 hsi = tex2D( HSLSampler, xy1 );
   float4 ret;

   // Scale and bias our components
   float h = frac( hsi.r + HueBias );
   float s = saturate( hsi.g * SatScale + SatBias );
   float i = saturate( hsi.b * ValScale  + ValBias );

   float h3 = 3 * h;
   float x = ( 2 * floor( h3 ) + 1 ) / 6;

   // Get our rgb components
   float r = ( 1 - s ) * i;
   float H = _rt3 * tan( ( h - x ) * _pi2 );
   float b = ( ( 3 + 3 * H ) * i - ( 1 + 3 * H ) * r ) / 2;
   float g = 3 * i - b - r;

   // Put them in right order
   if ( h3 < 1 )
   {
      ret = float4( g, b, r, hsi.a );
   }
   else if ( h3 < 2 )
   {
      ret = float4( r, g, b, hsi.a );
   }
   else
   {
      ret = float4( b, r, g, hsi.a );
   }

   return ret;
}

//--------------------------------------------------------------//
// Techniques
//--------------------------------------------------------------//
technique ApplyRGBCorrections
{
   pass P1 { PixelShader = compile PROFILE main(); }
}

technique T2
{
   pass RGBtoHSL
   <
      string Script = "RenderColorTarget0 = _hsl;";
   >
   {
      PixelShader = compile PROFILE RGBtoHSL();
   }

   pass HSLtoRGB
   {
      PixelShader = compile PROFILE HSLtoRGB();
   }
}
