/*
-------------------------------------------------------------------------
   File        : scopes.cl
   Description : OpenCL code for generating video analyses
   Author      : Rob
   Date        : Dec 2014

   Copyright (c) EditShare 2014.  All Rights Reserved
-------------------------------------------------------------------------
*/
#define kScopeHeight 256

const sampler_t sampler = CLK_FILTER_NEAREST;

__kernel void waveformY( __write_only image2d_t output, __read_only image2d_t input )
{
   uint w = get_image_width( input );
   uint h = get_image_height( input );

   int2 xy = ( int2 )( get_global_id( 0 ), 0 );

   if ( xy.x < w )
   {
      const float4 kLumaMultiplier = MAKEPIXEL( 0.3, 0.59, 0.11, 1.0 );

      uint luma[ kScopeHeight ];

      for ( uint i=0; i < kScopeHeight; ++i )
         luma[ i ]  = 0;

      float maxVal = ( float )( kScopeHeight - 1 );

      uint lastBrightness = 0;

      for ( ; xy.y < h; ++xy.y )
      {
         float4 pixel = read_imagef( input, sampler, xy ) * maxVal * kLumaMultiplier;

         uint brightness = ( uint )( ( RED( pixel ) + GREEN( pixel ) + BLUE( pixel ) ) + 0.5f );

         ++luma[ brightness ];

/*
         uint lower = min( brightness, lastBrightness );
         uint upper = max( brightness, lastBrightness ) + 1;
         uint count = upper - lower;

         while ( count-- )
            ++luma[ upper-- ];

         lastBrightness = brightness;
*/
      }
      xy.y = 0;

      for ( int y = kScopeHeight - 1; y >= 0; --y, ++xy.y )
      {
         float brightness = ( float ) luma[ y ] / ( float ) h;

         brightness = 1.0 - pow( 1.0 - brightness, 30 );

         write_imagef( output, xy, MAKEPIXEL( brightness, brightness, brightness, 1.0f ) );
      }
   }
}
//-------------------------------------------------------------------------
__kernel void waveformRGB( __write_only image2d_t output, __read_only image2d_t input )
{
   uint w = get_image_width( input );
   uint h = get_image_height( input );

   int2 xy = ( int2 )( get_global_id( 0 ), 0 );

   if ( xy.x < w )
   {
      float red[ kScopeHeight ];
      float green[ kScopeHeight ];
      float blue[ kScopeHeight ];

      for ( uint i=0; i < kScopeHeight; ++i )
      {
         red[ i ] = 0;
         green[ i ] = 0;
         blue[ i ] = 0;
      }

      float maxVal = ( float )( kScopeHeight - 1 );
      float inc = 50.0f / h;

      for ( ; xy.y < h; ++xy.y )
      {
         float4 pixel = ( read_imagef( input, sampler, xy ) * maxVal ) + 0.5f;

         red[ ( uint ) RED( pixel ) ]     += inc;
         green[ ( uint ) GREEN( pixel ) ] += inc;
         blue[ ( uint ) BLUE( pixel ) ]   += inc;
      }

      xy.y = 0;

      for ( int y = kScopeHeight - 1; y >= 0; --y, ++xy.y )
         write_imagef( output, xy, MAKEPIXEL( red[ y ], green[ y ], blue[ y ], 1.0f ) );
   }
}
//-------------------------------------------------------------------------
__kernel void waveformR( __write_only image2d_t output, __read_only image2d_t input )
{
   uint w = get_image_width( input );
   uint h = get_image_height( input );

   int2 xy = ( int2 )( get_global_id( 0 ), 0 );

   if ( xy.x < w )
   {
      float red[ kScopeHeight ];

      for ( uint i=0; i < kScopeHeight; ++i )
         red[ i ] = 0;

      float maxVal = ( float )( kScopeHeight - 1 );

      float inc = 50.0f / h;

      for ( ; xy.y < h; ++xy.y )
      {
         float4 pixel = read_imagef( input, sampler, xy );

         uint r = ( uint )( ( RED( pixel ) * maxVal ) + 0.5f );

         red[ r ] += inc;
      }

      for ( int y = kScopeHeight - 1; y >= 0; --y )
      {
         if ( red[ y ] )
         {
            red[ y ] = maxVal;
            break;
         }
      }

      xy.y = 0;

      for ( int y = kScopeHeight - 1; y >= 0; --y, ++xy.y )
         write_imagef( output, xy, MAKEPIXEL( red[ y ], 0.0f, 0.0f, 1.0f ) );
   }
}
//-------------------------------------------------------------------------
__kernel void waveformG( __write_only image2d_t output, __read_only image2d_t input )
{
   int2 xy = ( int2 )( get_global_id( 0 ), 0 );

   if ( xy.x < get_image_width( input ) )
   {
      uint h = get_image_height( input );

      uint temp[ kScopeHeight ];

      for ( uint i=0; i < kScopeHeight; ++i )
         temp[ i ] = 0;

      float maxVal = ( float )( kScopeHeight - 1 );

      uint last = 0;

      for ( ; xy.y < h; ++xy.y )
      {
         float4 pixel = read_imagef( input, sampler, xy );

         uint b = ( uint )( ( GREEN( pixel ) * maxVal ) + 0.5f );

         uint lower = min( b, last );
         uint upper = max( b, last ) + 1;
         uint count = upper - lower;

         while ( count-- )
            ++temp[ upper-- ];

         last = b;
      }

/*
      for ( ; xy.y < h; ++xy.y )
      {
         float4 pixel = read_imagef( input, sampler, xy );

         uint idx = ( uint )( ( GREEN( pixel ) * maxVal ) + 0.5f );

         ++temp[ idx ];
      }
*/
      xy.y = xy.x;
      xy.x = kScopeHeight - 1;

      float multiplier = 50.0f / h;

      for ( uint x=0; x < kScopeHeight; ++x, --xy.x )
         write_imagef( output, xy, MAKEPIXEL( 0.0f, temp[ x ] * multiplier, 0.0f, 1.0f ) );
   }
}
//-------------------------------------------------------------------------
__kernel void waveformB( __write_only image2d_t output, __read_only image2d_t input )
{
   uint w = get_image_width( input );
   uint h = get_image_height( input );

   int2 xy = ( int2 )( get_global_id( 0 ), 0 );

   if ( xy.x < w )
   {
      uint temp[ kScopeHeight ];

      for ( uint i=0; i < kScopeHeight; ++i )
         temp[ i ] = 0;

      float maxVal = ( float )( kScopeHeight - 1 );

      uint last = 0;

      for ( ; xy.y < h; ++xy.y )
      {
         float4 pixel = read_imagef( input, sampler, xy );

         uint idx = ( uint )( ( BLUE( pixel ) * maxVal ) + 0.5f );

         uint lower = min( idx, last );
         uint upper = max( idx, last ) + 1;
         uint count = upper - lower;

         while ( count-- )
            ++temp[ upper-- ];

         last = idx;
      }

      xy.y = kScopeHeight - 1;

      float multiplier = 50.0f / h;

      for ( uint y=0; y < kScopeHeight; ++y, --xy.y )
         write_imagef( output, xy, MAKEPIXEL( 0.0f, 0.0, temp[ y ] * multiplier, 1.0f ) );
   }
}

//-------------------------------------------------------------------------
__kernel void vectorscope( global uint* output, __read_only image2d_t input )
{
   uint w = get_image_width( input );
   uint h = get_image_height( input );

   int2 xy = ( int2 )( get_global_id( 0 ), get_global_id( 1 ) );

   if ( xy.x < w )
   {
      const float4 cbMultiplier = MAKEPIXEL( -0.16874f, -0.33126f, 0.5f, 1.0f );
      const float4 crMultiplier = MAKEPIXEL( 0.5f, -0.41869f, -0.08131f, 1.0f );

      float4 pixel = read_imagef( input, sampler, xy );

      float4 tmp1 = pixel * crMultiplier;
      float4 tmp2 = pixel * cbMultiplier;

      float u = ( RED( tmp1 ) + GREEN( tmp1 ) + BLUE( tmp1 ) ) * 255.0f;
      float v = ( RED( tmp2 ) + GREEN( tmp2 ) + BLUE( tmp2 ) ) * 202.0f;

      int2 outPos = ( int2 )( ( int )( 128.5f + v ), ( int )( 128.5f - u ) );

      atom_inc( &output[ outPos.y * 256 + outPos.x ] );
   }
}

//-------------------------------------------------------------------------
__kernel void vectorscope2( __write_only image2d_t output, global uint* input, float scale )
{
   uint w = get_image_width( output );
   uint h = get_image_height( output );

   int2 xy = ( int2 )( get_global_id( 0 ), get_global_id( 1 ) );

   if ( ( xy.x < w ) && ( xy.y < h ) )
   {
      float luminance = input[ xy.y * 256 + xy.x ];
      luminance *= scale;

      write_imagef( output, xy, MAKEPIXEL( luminance, luminance, luminance, 1 ) );
   }
}

//-------------------------------------------------------------------------
__kernel void zeromem( global uint* mem )
{
   mem[ get_global_id( 0 ) ] = 0;
}

//-------------------------------------------------------------------------
__kernel void rotate90( __write_only image2d_t output, __read_only image2d_t input )
{
   uint w = get_image_width( input );
   uint h = get_image_height( input );

   int2 xy = ( int2 )( get_global_id( 0 ), get_global_id( 1 ) );

   if ( ( xy.x < w ) && ( xy.y < h ) )
   {
      float4 pixel = read_imagef( input, sampler, xy );
      write_imagef( output, ( int2 )( xy.y, xy.x ), pixel );
   }
}

