//--------------------------------------------------------------//
// Selective Colour Correction 2
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "SCC";
   string Category    = "Colour";
> = 0;

//--------------------------------------------------------------
// Params
//--------------------------------------------------------------
float4 BlackLevel;
float4 GreyLevel;
float4 WhiteLevel;
float4 ShadowTintColour;
float4 MidTintColour;
float4 HighTintColour;
float  Saturation;
float4 Gamma;
float4 Contrast;
float4 Brightness;
float4 Gain;
float  HueBias;
float  SatBias;
float  ValBias;
float  SatScale;
float  ValScale;
float4 SelectionColour;
float4 SelectionTolerance;
float4 SelectionSoftness;
bool   SelectionInvert;
bool   SelectionReveal;

float _pi     = 3.14159;
float _pi2    = 6.2831853;
float _offset = 0.5 / 256.0;
float _scale  = 255.0 / 256.0;
float _rt3    = 1.0 / sqrt( 3.0 );

float _oneSixth   = 1.0 / 6.0;
float _twoSixths  = 2.0 / 6.0;
float _oneHalf    = 0.5;
float _fourSixths = 4.0 / 6.0;
float _fiveSixths = 5.0 / 6.0;

#define HUE_IDX 0
#define SAT_IDX 1
#define VAL_IDX 2

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture Input;
texture _lut;
texture _hsl;
texture _one, _two, _three, _four;

sampler InputSampler = sampler_state { Texture = <Input>; };
sampler LUTSampler   = sampler_state { Texture = <_lut>; };
sampler HSLSampler   = sampler_state { Texture = <_hsl>; };

sampler Sampler1     = sampler_state { Texture = <_one>; };
sampler Sampler2     = sampler_state { Texture = <_two>; };
sampler Sampler3     = sampler_state { Texture = <_three>; };
sampler Sampler4     = sampler_state { Texture = <_four>; };

bool allPositive( float3 pixel )
{
   return pixel.r > 0 && pixel.g > 0 && pixel.b > 0; 
}

//---------------------------------------------------------------------------------------//
// Logic
//---------------------------------------------------------------------------------------//
float4 ApplyRGBCorrections_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   // Read the source image pixel
   float4 src = tex2D( InputSampler, xy1 );
   float4 hsv = tex2D( HSLSampler, xy1 );
   float4 ret = src;

   if ( SelectionReveal == false )
   {
      // Do balancing
      float greyAverage = ( GreyLevel.r + GreyLevel.g + GreyLevel.b ) / 3.0;
      ret -= BlackLevel;
      ret /= ( WhiteLevel - BlackLevel );
      ret -= GreyLevel;
      ret += greyAverage;
      ret = clamp( ret, 0.0, 1.0 );

      // Calc its brightness (so that it can be treated as a shadow/midtone/highlight as appropriate)
      float lum = ( ret.r + ret.g + ret.b ) / 3.0;

      // Apply the shadow/midtone/highlight tints
      float2 lutPos   = float2( ( lum * _scale ) + _offset, 0.5 );
      float4 lutPixel = tex2D( LUTSampler, lutPos );
      ret += ShadowTintColour * lutPixel.r;
      ret += MidTintColour    * lutPixel.g;
      ret += HighTintColour   * lutPixel.b;

      float4 avg = ( ret.r + ret.g + ret.b ) / 3.0;
      ret = avg + ( ( ret - avg ) * Saturation );

      // Do brightness, gamma & contrast logic
      ret = ( ( ( ( pow( saturate( ret ), 1 / Gamma ) * Gain ) + Brightness ) - 0.5 ) * Contrast ) + 0.5;
   }
   else
      ret = float4( 1,1,1,1 );

   // Calc difference between current pixel and specified key-colour

   float4 diff = abs( hsv - SelectionColour );

   if ( diff[ HUE_IDX ] > 0.5 )
      diff[ HUE_IDX ] = 1.0 - diff[ HUE_IDX ];

   // Work out how transparent/opaque the corrected pixel will be

   float mix = 0;

   if ( allPositive( SelectionTolerance.rgb + SelectionSoftness.rgb - diff.rgb ) )
   {
      if ( allPositive( SelectionTolerance.rgb - diff.rgb ) )
         mix = 1;
      else
      {
         diff -= SelectionTolerance;
         diff /= SelectionSoftness;
         mix = 1.0 - ( max( diff[ HUE_IDX ], max( diff[ SAT_IDX ], diff[ VAL_IDX ] ) ) );
      }
   }

   ret.a = lerp( mix, 1.0 - mix, SelectionInvert );

   return ret;
}

//--------------------------------------------------------------//
// RGBtoHSL
//--------------------------------------------------------------//
float4 RGBtoHSL_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 hsv = 0;
   float4 rgba = tex2D( InputSampler, xy1 );

   float maxComponentVal = max( max( rgba.r, rgba.g ), rgba.b );
   float minComponentVal = min( min( rgba.r, rgba.g ), rgba.b );
   float componentRange  = maxComponentVal - minComponentVal;

   hsv[ VAL_IDX ] = maxComponentVal;
   hsv[ SAT_IDX ] = componentRange / maxComponentVal;

   if ( hsv[ SAT_IDX ] != 0 )
   {
      if ( rgba.r == maxComponentVal )
      {
         hsv[ HUE_IDX ] = ( rgba.g - rgba.b ) / componentRange;
      }
      else if ( rgba.g == maxComponentVal )
      {
         hsv[ HUE_IDX ] = 2 + ( ( rgba.b - rgba.r ) / componentRange );
      }
      else
      {
         hsv[ HUE_IDX ] = 4 + ( ( rgba.r - rgba.g ) / componentRange );
      }

      hsv[ HUE_IDX ] *= _oneSixth;

      if ( hsv[ HUE_IDX ] < 0 )
         hsv[ HUE_IDX ] += 1.0;
   }

   return hsv;
}

//--------------------------------------------------------------
// ApplyHSVCorrections
//
// Convert to HSV, apply the corrections then convert back th RGB
//--------------------------------------------------------------
float4 ApplyHSVCorrections_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, xy1 );
   float4 ret;

   float i  = ( src.r + src.g + src.b ) / 3.0;
   float r  = src.r;
   float g  = src.g;
   float b  = src.b;
   float rg = r - g;
   float rb = r - b;

   // Hue
   float h = acos( ( rg + rb ) / ( 2 * sqrt( rg * rg + rb * ( g - b ) ) ) ) / _pi2;

   if ( b > g )
      h = 1 - h;

   // Saturation
   float s = 1 - min( min( r, g ), b ) / i;

   // Scale and bias our components
   h = frac( h + HueBias );
   s = saturate( s * SatScale + SatBias );
   i = saturate( i * ValScale  + ValBias );

   float h3 = 3 * h;
   float x = (2 * floor(h3) + 1) / 6;

   // Get our rgb components
   float H = _rt3 * tan( ( h - x ) * _pi2 ) * 3;
   r = ( 1 - s ) * i;
   b = ( ( 3 + H ) * i - ( 1 + H ) * r ) / 2;
   g = 3 * i - b - r;

   // Put them in right order

   if ( h3 < 1 )
   {
      ret = float4( g, b, r, src.a );
   }
   else if ( h3 < 2 )
   {
      ret = float4( r, g, b, src.a );
   }
   else
   {
      ret = float4( b, r, g, src.a );
   }

   return ret;
}
//--------------------------------------------------------------//
float4 CompositeOne_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 source     = tex2D( InputSampler, xy1 );
   float4 corrected1 = tex2D( Sampler1, xy1 );

   float4 ret = lerp( source, corrected1, corrected1.a );
   ret.a = 1.0;

   return ret;
}
//--------------------------------------------------------------//
float4 CompositeTwo_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 source     = tex2D( InputSampler, xy1 );
   float4 corrected1 = tex2D( Sampler1, xy1 );
   float4 corrected2 = tex2D( Sampler2, xy1 );

   float4 ret = lerp( source, corrected1, corrected1.a );
   ret = lerp( ret, corrected2, corrected2.a );
   ret.a = source.a;

   return ret;
}
//--------------------------------------------------------------//
float4 CompositeThree_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 source     = tex2D( InputSampler, xy1 );
   float4 corrected1 = tex2D( Sampler1, xy1 );
   float4 corrected2 = tex2D( Sampler2, xy1 );
   float4 corrected3 = tex2D( Sampler3, xy1 );

   float4 ret = lerp( source, corrected1, corrected1.a );
   ret = lerp( ret, corrected2, corrected2.a );
   ret = lerp( ret, corrected3, corrected3.a );
   ret.a = source.a;

   return ret;
}
//--------------------------------------------------------------//
float4 CompositeFour_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 source     = tex2D( InputSampler, xy1 );
   float4 corrected1 = tex2D( Sampler1, xy1 );
   float4 corrected2 = tex2D( Sampler2, xy1 );
   float4 corrected3 = tex2D( Sampler3, xy1 );
   float4 corrected4 = tex2D( Sampler4, xy1 );

   float4 ret = lerp( source, corrected1, corrected1.a );
   ret = lerp( ret, corrected2, corrected2.a );
   ret = lerp( ret, corrected3, corrected3.a );
   ret = lerp( ret, corrected4, corrected4.a );
   ret.a = source.a;

   return ret;
}

//--------------------------------------------------------------//
// Techniques
//--------------------------------------------------------------//
technique ApplyRGBCorrections { pass P1 { PixelShader = compile PROFILE ApplyRGBCorrections_main(); } }
technique ApplyHSVCorrections { pass P1 { PixelShader = compile PROFILE ApplyHSVCorrections_main(); } }
technique RGBtoHSL            { pass P1 { PixelShader = compile PROFILE RGBtoHSL_main(); } }
technique CompositeOne        { pass P1 { PixelShader = compile PROFILE CompositeOne_main(); } }
technique CompositeTwo        { pass P1 { PixelShader = compile PROFILE CompositeTwo_main(); } }
technique CompositeThree      { pass P1 { PixelShader = compile PROFILE CompositeThree_main(); } }
technique CompositeFour       { pass P1 { PixelShader = compile PROFILE CompositeFour_main(); } }

